#!/bin/sh
set -e

# Check we have the necessary arguments.
if [ "$#" -lt 1 ]; then
    echo 'Need mail for letsencrypt, mail domain, and optionally old server IP.'
    false
fi
mail="$1"
mail_domain="$2"
old_server="$3"

config_tree_prefix="${HOME}/config/buster"
echo "postfix postfix/main_mailer_type string 'Internet Site'" | debconf-set-selections
echo "postfix postfix/mailname string ${mail_domain}" | debconf-set-selections
./install_for_target.sh mail
./copy_dirtree.sh "${config_tree_prefix}/etc_files" "" mail
nft -f /etc/nftables.conf

# Rebuild aliases DB from /etc/aliases
newaliases

# Update config files without overwriting defaults.
cat "${config_tree_prefix}/other_files/append_postfix_main.cf" >> /etc/postfix/main.cf
cat "${config_tree_prefix}/other_files/append_postfix_master.cf" >> /etc/postfix/master.cf
cat "${config_tree_prefix}/other_files/append_opendkim.conf" >> /etc/opendkim.conf

# Set up letsencrypt certificate.  We need this for STARTTLS on port
# 25/SMTP (some mail servers refuse delivering mails here if no
# STARTTLS available) and transport-layer TLS on port 465 (for
# user-to-server SMTPS)
# TODO: Is it auto-renewed?
certbot certonly --standalone --agree-tos --no-eff-email -m "${mail}" -d "$(hostname -f)"

# For if FQDN != mail domain name.
sed -i "s/REPLACE_maildomain_ECALPER/${mail_domain}/g" /etc/mailutils.conf
sed -i 's/REPLACE_mail_domain_ECALPER/${mail_domain}/g' /etc/postfix/main.cf

# OpenDKIM setup.
selector=$(hostname)$(date +%Y%m%d)
opendkim-genkey -d "${mail_domain}" -D /etc/dkimkeys -s "${selector}"
sed -i "s/REPLACE_hostname_ECALPER/$(hostname -f)/g" /etc/opendkim.conf
sed -i "s/REPLACE_selector_ECALPER/${selector}/g" /etc/opendkim.conf

# Dovecot sieve filtering via LMTP.  Without this, mail only gets
# delivered to /var/mail/…, with it /var/mail/… remains the fallback
# inbox, but all else is sieve-filtered to ~/mail/.
cp "${config_tree_prefix}/other_files/dovecot.sieve" /home/plom/.dovecot.sieve
chown plom:plom /home/plom/.dovecot.sieve

# In addition to our postfix server receiving mails, we funnel mails from a
# POP3 account into dovecot via fetchmail. It might make sense to adapt the
# ~/.dovecot.sieve to move mails targeted to the fetched mail account to their
# own mbox.
cp "${config_tree_prefix}/other_files/fetchmailrc" /home/plom/.fetchmailrc
chown plom:plom /home/plom/.fetchmailrc
chmod 0700 /home/plom/.fetchmailrc

# Pingmail setup.
cp "${config_tree_prefix}/other_files/pingmailrc" /home/plom/.pingmailrc
chown plom:plom /home/plom/.pingmailrc
su -lc "cd && git clone https://plomlompom.com/repos/clone/pingmail" plom

# To allow IMAPS access.
echo "ssl_cert = </etc/letsencrypt/live/$(hostname -f)/fullchain.pem" > /etc/dovecot/conf.d/99-ssl-certs.conf
echo "ssl_key = </etc/letsencrypt/live/$(hostname -f)/privkey.pem" >> /etc/dovecot/conf.d/99-ssl-certs.conf
password=$(pwgen -s 100 1)
echo "plom:${password}" | chpasswd

# Get old mail data, shutdown old postfix server.
if [ "${old_server}" != "" ]; then
  cp "${config_tree_prefix}/setup_scripts/prepare_to_meet_server.sh" /home/plom/
  #chown plom:plom /home/plom/prepare_to_meet_server.sh
  su -lc "./prepare_to_meet_server.sh ${old_server}" plom
  read -p'Hit Enter when you are done.' ignore
  rm /home/plom/prepare_to_meet_server.sh
  su -lc "scp plom@${old_server}:.dovecot.sieve ~" plom
  su -lc "scp plom@${old_server}:.fetchmailrc ~" plom
  su -lc "scp plom@${old_server}:.pingmailrc ~" plom
  su -lc "ssh -t plom@${old_server} \"su -lc 'service postfix stop'\"" plom
  #su -lc "ssh plom@${old_server} \"su -lc 'service fetchmail_old_account stop'\"" plom
  su -lc "ssh -t plom@${old_server} \"su -lc 'service fetchmail stop'\"" plom
  cp "${config_tree_prefix}/setup_scripts/mirror_dir.sh" /home/plom/
  su -lc "./mirror_dir.sh ${old_server} /home/plom/mail" plom
  rm /home/plom/mirror_dir.sh
  touch /var/mail/plom
  chown plom:mail /var/mail/plom
  chmod 0600 /var/mail/plom
  su -lc "scp plom@${old_server}:/var/mail/plom /var/mail/plom" plom
fi

# Start everything anew to ensure new configurations.
service opendkim restart
service postfix restart
service dovecot restart

# Pingmail and fetchmail have some systemd timers waiting. To let systemd
# know about them, do this.
systemctl daemon-reload
#systemctl enable --now fetchmail_old_account.timer
#systemctl enable --now pingmail.timer

# Final advice to user.
echo "To put into DNS:"
cat "/etc/dkimkeys/${selector}.txt"
echo "If subdomain, append .subdomain to _domainkeys!"
echo "Also ensure DMARC record of 'v=DMARC1; p=none; rua=mailto:plom+dmarc@plomlompom.com;' as TXT entry at _dmarc or, if subdomain, _dmarc.subdomain"
echo "Also ensure SPF record of 'v=spf1 mx -all' as TXT entry at @ or subdomain"
echo "Also ensure reverse DNS lookup for our IP points to $(hostname -f)"
echo "Also ensure MX record of priority 10 for @ or subdomain pointing to $(hostname -f)"
echo "IMAPS password for user plom is: ${password}"

# todo just for proper mail /sending/:
# * how to check IP safety
# https://talosintelligence.com/reputation_center/lookup?search=$IP
# http://www.anti-abuse.org/multi-rbl-check-results/?host=
# https://www.dnsbl.info/dnsbl-database-check.php
# note that none of these catch the IPs that gmx etc. reject
